#include <stdio.h>
#include <stdlib.h>
#include "iocrema.h"

#define VUOTO '.'
#define SIMBOLO1 'X'
#define SIMBOLO2 'O'


/* Pone tutti gli elementi di tabella (3x3) a VUOTO*/
void Inizializza(int (*tabella)[3]);

/* Torna 0 se si puo' continuare a giocare
   Torna 1 se ha vinto il giocatore 1
   Torna 2 se ha vinto il giocatore 2
   Torna 3 se il gioco  finito pari
*/
int GiocoFinito(int (*tabella)[3]);
int Vittoria(int (*tabella)[3]);
int Pari(int (*tabella)[3]);

/* Legge la mossa effettuata dal giocatore "turno", con
   la seguente codifica della posizione delle caselle:

   1  2  3
   4  5  6
   7  8  9

   Controlla che la mossa sia valida e in caso contrario ripete la
   lettura della mossa

   ESTENSIONE: Oppure torna 0 se si vuole interrompere la partita
    ESTENSIONE: Suggerisce le mosse possibili o la mossa migliore
 */
int LeggiMossa(int (*tabella)[3], int turno);

/* Pone la mossa "mossa" del giocatore "turno" nella "tabella" */
void Aggiorna(int (*tabella)[3], int turno, int mossa);

/* Visualizza la situazione di gioco attuale
   Visualizza le istruzioni di gioco */
void Visualizza(int (*tabella)[3], int turno);

/* Visualizza il messaggio di termine partita
    In "situazione" viene passato il valore
    con la codifica usata in GiocoFinito
*/
void VisualizzaFine(int situazione);


int main()
{
    int board[3][3];
    int turno; /* id del giocatore che deve muovere */
    int FINE;
    
    /* inizializza board */
    Inizializza(board);
    turno = 1;

    FINE = GiocoFinito(board);
    Visualizza(board, turno);
    while(!FINE)  
    {
      /* esce dal ciclo se 
	 - interruzione: FINE = -1
	 - partita vinta: FINE = 1 o FINE = 2
	 - partita patta: FINE = 3
	 resta nel ciclo se:
	 - mossa possibile: FINE = 0
       */

        /* Gioco */
        int mossa;
	
        /* Ritorna la codifica della casella
            nella quale il giocatore "turno"
            mette il simbolo */
        mossa = LeggiMossa(board, turno);
	if (mossa == 0)
	  FINE = -1; /* interruzione partita */
	else {
	  /* Aggiorna la tabella aggiungendo
	     sulla casella "mossa" il simbolo
	     del giocatore "turno" */
	  Aggiorna(board, turno, mossa);
	  turno = (turno==1 ? 2 : 1);
	  
	  /* visualizza la mossa fatta */
	  Visualizza(board, turno);
	  FINE = GiocoFinito(board);
	}
    }
    VisualizzaFine(FINE);

    return 0;
}

void Inizializza(int (*tabella)[3])
{
    /* Mette tutte le caselle a 0 */
  
    int i,j;
    for (i=0; i<3; i++)
        for (j=0; j<3; j++)
	  tabella[i][j] = 0;

    /* DEBUG*/
  /*
    tabella[0] = 1; tabella[1] = 2; tabella[2] = 1;
    tabella[3] = 0; tabella[4] = 2; tabella[5] = 0;
    tabella[6] = 0; tabella[7] = 2; tabella[8] = 0;
  */
}

/* Valuta la situazione in tabella e restituisce:
    0 se si puo' continuare a giocare
    1 se ha vinto il giocatore 1
    2 se ha vinto il giocatore 2
    3 se il gioco  finito pari
*/
int GiocoFinito(int (*tabella)[3]) {
    int caso;
    /* torna 0 se nessuno ha ancora vinto, altrimenti torna
    l'id del vincitore */
    caso = Vittoria(tabella);
    if (caso==0){
            /* torna 3 se il gioco e' pari, 0 altrimenti */
        caso = Pari(tabella);
    }
    return caso;
}

int Vittoria(int (*tabella)[3]) {
  
  int i;
  /* controlla le righe */
  for (i=0; i<3; i++)
    if (tabella[i][0]==tabella[i][1] && tabella[i][0]==tabella[i][2])
      return tabella[i][0];
  /* controlla le colonne */
  for (i=0; i<3; i++)
    if (tabella[0][i]==tabella[1][i] && tabella[0][i]==tabella[2][i])
      return tabella[0][i];
  /* controlla la prima diagonale */
  if (tabella[0][0]==tabella[1][1] && tabella[0][0]==tabella[2][2])
      return tabella[0][0];
  /* controlla la seconda diagonale */
  if (tabella[0][2]==tabella[1][1] && tabella[0][2]==tabella[2][0])
      return tabella[0][2];

  /* Se arriva qui, nessuno ha vinto */
  return 0;
}

int Pari(int (*tabella)[3]) {
  int i, j;
  int pari = 1;
  
  for (i=0; i<3; i++)
    for (j=0; j<3; j++)
      pari = pari && (tabella[i][j] != 0);

  if (pari)
    return 3; /* codifica per il caso di partita patta */
  else
    return 0; /* codifica per il caso in cui sia possibile muovere */
  
}


/* Legge la mossa effettuata dal giocatore "turno", con
   la seguente codifica della posizione delle caselle:

   1  2  3    [0][0]  [0][1]  [0][2]
   4  5  6    [1][0]  [1][1]  [1][2]
   7  8  9    [2][0]  [2][1]  [2][2]

   Controlla che la mossa sia valida e in caso contrario ripete la
   lettura della mossa

   ESTENSIONE: Oppure torna 0 se si vuole interrompere la partita
   ESTENSIONE: Suggerisce le mosse possibili o la mossa migliore
 */
int LeggiMossa(int (*tabella)[3], int turno) {
  
  int mossa;
  do {
    StampaStringa("Mossa giocatore ");
    StampaIntero(turno);
    StampaStringa(": ");
    mossa = LeggeIntero();
  } while (tabella[(mossa-1)/3][(mossa-1)%3] != 0);
  return mossa;
}

/* Pone la mossa "mossa" del giocatore "turno" nella "tabella" */
void Aggiorna(int (*tabella)[3], int turno, int mossa) {
  tabella[(mossa-1)/3][(mossa-1)%3] = turno;
}

/* Visualizza la situazione di gioco attuale
   Visualizza le istruzioni di gioco */
void Visualizza(int (*tabella)[3], int turno)
{
  /* Visualizza il simbolo corrispondente al contenuto
     di ciascuna casella di "tabella" :
     se 0, VUOTO
     se 1, SIMBOLO1
     se 2, SIMBOLO2
  */
  int i, j;
    
  ACapo();
  StampaStringa("Digitare 0 per interrompere la partita");
  ACapo();
  StampaStringa("oppure il numero corrispondente alla mossa desiderata:");
  ACapo();
  StampaStringa("1  2  3");
  ACapo();
  StampaStringa("4  5  6");
  ACapo();
  StampaStringa("7  8  9");
  ACapo();
  StampaStringa("---------");
  ACapo();

  for (i=0; i<3; i++)
    {
      for (j=0; j<3; j++)
	{
	  StampaStringa(" ");
	  /* tabella e' una matrice 3x3 */
	  switch(tabella[i][j])
	    {
	    case 0:
	      StampaCarattere(VUOTO, 1);
	      break;
	    case 1:
	      StampaCarattere(SIMBOLO1, 1);
	      break;
	    case 2:
	      StampaCarattere(SIMBOLO2, 1);
	      break;
	    default:
	      StampaStringa("ERRORE: valore in tabella non valido");
	      ACapo();
	    }
	  StampaStringa(" ");
	} /* for j*/
      ACapo();
    } /* for i*/

  StampaStringa("---------");
  ACapo();
  
}

/* Visualizza il messaggio di termine partita
    In "situazione" viene passato il valore
    con la codifica usata in GiocoFinito
*/
void VisualizzaFine(int situazione) {
  
  switch(situazione) {
  case -1: 
    StampaStringa("Partita Interrotta");
    ACapo();
    break;

  case 1:
  case 2:
    StampaStringa("Ha vinto il giocatore ");
    StampaIntero(situazione);
    StampaStringa(". CONGRATULAZIONI!");
    ACapo();
    break;
    
  case 3:
    StampaStringa("Partita pari. Bravi entrambi!");
    ACapo();
    break;
  }
}



